-----------------------------------------------------------------------------------------------
-- Client Lua Script for ZenRadar
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 require "Window"
 require "QuestLib"
 
-----------------------------------------------------------------------------------------------
-- ZenRadar Module Definition
-----------------------------------------------------------------------------------------------
local ZenRadar = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local cSlashCommand = "zr"
local cAddonName = "ZenRadar"
local cHasConfigurationButton = true
local cDeleteXml = true

local cLocaleKeyHarvestMining = "HarvestRequiredMining"
local cLocaleKeyHarvestSurvivalist = "HarvestRequiredSurvivalist"
local cLocaleKeyHarvestRelic = "HarvestRequiredRelic"
local cLocaleKeyHarvestFarming = "HarvestRequiredFarming"
local cLocaleKeyAffiliationSettlerResource = "AffiliationSettlerResource"
local cLocaleKeyNameExplorerDiscovery = "NameExplorerDiscovery"
local cLocaleKeyNameExplorerTrail = "NameExplorerTrail"

local cDisplayTypeMiner = "Node"
local cDisplayTypeSurvivalist = "Tree"
local cDisplayTypeRelic = "Relic"
local cDisplayTypeFarming = "Herb"

local cDisplayTypeMail = "Mail"
local cDisplayTypeSave = "Save"
local cDisplayTypeTaxi = "Taxi"
local cDisplayTypeWorkBench = "Craft"
local cDisplayTypeCostume = "Costume"
local cDisplayTypeBank = "Bank"
local cDisplayTypeRep = "Rep"
local cDisplayTypeHouse = "House"
local cDisplayTypeRecipe = "Recipe"

local cDisplayTypeTarget = "Target"
local cDisplayTypeLore = "Lore"
local cDisplayTypeCollect = "Collect"
local cDisplayTypeNewQuest = "Quest"
local cDisplayTypeQuestObj = "Objective"
local cDisplayTypePathQuestObj = "Path"
local cDisplayTypeDailyQuest = "Daily"
local cDisplayTypeScienceScan = "Scientist"
local cDisplayTypeScienceMat = "Raw"
local cDisplayTypeExplorerDiscovery = "Explorer"
local cDisplayTypeExplorerTrail = "Trail"
local cDisplayTypeSettlerResource = "Settler"

local cDisplayTypeWhiteList = "White"

local cDistanceMinExplorerTrail = 30

local cXmlFileName = "ZenRadar.xml"
local cMainWndClass = "ZenRadarForm"
local cCfgWndClass = "ZenRadarConfigForm"
local cMainWndTrackedUnits = "TrackedUnits"
local cMainWndSettingBtn = "BtnConfig"
local cMainWndLockBtn = "ChLockRadar"
local cMainWndWhiteBtn = "BtnAddWhiteList"
local cMainWndBlackBtn = "BtnAddBlackList"

local cCfgWndCheckData = "ChData"
local cCfgWndCheckMining = "ChMining"
local cCfgWndCheckSurvival = "ChSurvival"
local cCfgWndCheckRelic = "ChRelic"
local cCfgWndCheckHerb = "ChHerb"
local cCfgWndCheckHarvest = "ChHarvest"
local cCfgWndCheckCollect = "ChCollect"

local cCfgWndCheckMail = "ChMail"
local cCfgWndCheckSave = "ChSave"
local cCfgWndCheckTaxi = "ChTaxi"
local cCfgWndCheckWorkBench = "ChWorkBench"
local cCfgWndCheckCostume = "ChCostume"

local cCfgWndCheckNewQuest = "ChNewQuest"
local cCfgWndCheckQuestObj = "ChQuestObj"
local cCfgWndCheckPathQuestObj = "ChPathQuestObj"
local cCfgWndCheckDailyQuest = "ChDaily"
local cCfgWndCheckScientist = "ChScan"
local cCfgWndCheckScientistRaw = "ChRaw"
local cCfgWndCheckExplorer = "ChExplorer"
local cCfgWndCheckExplorerTrail = "ChExplorerTrail"
local cCfgWndCheckSettler = "ChSettler"

local cCfgWndCheckAutoHideNoEntities = "ChAutoHideNoEntities"
local cCfgWndCheckAutoHideInCombat = "ChAutoHideInCombat"
local cCfgWndCheckEnableTrackMaster = "ChEnableTrackMaster"
local cCfgWndCheckEnableTrackMasterClosest = "ChEnableTrackMasterClosest"
local cCfgWndCheckEnableTrackMasterSelected = "ChEnableTrackMasterSelected"

local cCfgWndCheckLockRadar = "ChLockRadar"
 
local cCfgWndTextBlackListFilter = "txtFilterBlkList"
local cCfgWndTextWhiteListFilter = "txtFilterWhtList"

-----------------------------------------------------------------------------------------------
-- Libraries
-----------------------------------------------------------------------------------------------
local L = Apollo.GetPackage("Gemini:Locale-1.0").tPackage:GetLocale("ZenRadar", true)
 
-----------------------------------------------------------------------------------------------
-- Settings
-----------------------------------------------------------------------------------------------
local cUserSettingMainWindowLocation = "MainWindowLocation"
local cShowDataSetting = "ShowData"
local cShowCollectSetting = "ShowCollect"
local cShowMiningSetting = "ShowMining"
local cShowSurvivalSetting = "ShowSurvival"
local cShowRelicSetting = "ShowRelic"
local cShowHerbSetting = "ShowHerb"
local cShowHarvestSetting = "ShowHarvest"
local cShowRawSetting = "ShowRaw"

local cShowMailSetting = "ShowMail"
local cShowSaveSetting = "ShowSave"
local cShowTaxiSetting = "ShowTaxi"
local cShowWorkBenchSetting = "ShowWorkBench"
local cShowCostumeSetting = "ShowCostume"
local cShowBankSetting = "ShowBank"
local cShowRepSetting = "ShowRep"

local cShowNewQuestSetting = "ShowNewQuest"
local cShowNewQuestOptSetting = "ShowNewQuestOpt"

local cShowQuestObjSetting = "ShowQuestObj"
local cShowPathQuestObjSetting = "ShowPathQuestObj"
local cShowDailySetting = "ShowDaily"
local cShowTargetSetting = "ShowTarget"
local cShowScanSetting = "ShowScan"
local cShowExplorerSetting = "ShowExplorer"
local cShowExplorerTrailSetting = "ShowExplorerTrail"
local cShowSettlerSetting = "ShowSettler"

local cEnableTrackMasterSetting = "EnableTrackMaster"
local cEnableTrackMasterClosestSetting = "EnableTrackMasterClosest"
local cEnableTrackMasterSelectedSetting = "EnableTrackMasterSelected"
local cTrackMasterLineNo = "TrackMasterLineNo"

local cAutoHideNoEntitiesSetting = "AutoHideNoEntities"
local cAutoHideInCombatSetting = "AutoHideInCombat"
local cHideRadarSetting = "Hide Radar"
local cHideButtonsSetting = "Hide Buttons"

local cLockRadarSetting = "LockRadar"

local cBlackListFilterSetting = "BlackListFilter"
local cWhiteListFilterSetting = "WhiteListFilter"

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ZenRadar:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	return o
end

function ZenRadar:Init()
	local tDependencies = {	}
    Apollo.RegisterAddon(self, cHasConfigurationButton, cAddonName, tDependencies)
	
	self.Settings = {}
	self.Settings[cAutoHideNoEntitiesSetting] = false
	self.Settings[cAutoHideInCombatSetting] = false
	self.Settings[cHideRadarSetting] = false
	self.Settings[cShowDataSetting] = true
	self.Settings[cShowCollectSetting] = true
	self.Settings[cShowNewQuestSetting] = true
	self.Settings[cShowNewQuestOptSetting] = true
	self.Settings[cShowQuestObjSetting] = true
	self.Settings[cShowTargetSetting] = true
	self.Settings[cShowPathQuestObjSetting] = true
	self.Settings[cShowSaveSetting] = true
	self.Settings[cShowTaxiSetting] = true
	
	self.aAllUnits = {}
	self.aUnitsToCheckOnNextUpdate = {}
	self.aUnitsCheckValidityOnUpdate = {}
	self.aBlackListFilters = {}
	self.aWhiteListFilters = {}
	self.aAllQuests = {}
	self.aAllPathQuests = {}
end
 
function ZenRadar:OnLoad()
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
	Apollo.RegisterEventHandler("UnitActivationTypeChanged", "OnUnitChanged", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnUnitCombat", self)

	--- Quest Stuff
	Apollo.RegisterEventHandler("QuestStateChanged", "OnQuestStateChanged", self)
	Apollo.RegisterEventHandler("QuestObjectiveUpdated", "OnQuestObjectiveUpdated", self)
	
	-- Path Stuff
	Apollo.RegisterEventHandler("PlayerPathMissionActivate", "OnPlayerPathMissionActivate", self)
	Apollo.RegisterEventHandler("PlayerPathMissionUpdate", 	"OnPlayerPathMissionUpdate", self)
	Apollo.RegisterEventHandler("PlayerPathMissionDeactivate", "OnPlayerPathMissionDeactivate", self)
	Apollo.RegisterEventHandler("PlayerPathMissionComplete", "OnPlayerPathMissionComplete", self)
	
	-- Challenge stuff and events
	Apollo.RegisterEventHandler("PublicEventStart", 			"OnPublicEventUpdate", self)
	Apollo.RegisterEventHandler("PublicEventObjectiveUpdate", 	"OnPublicEventUpdate", self)
	Apollo.RegisterEventHandler("PublicEventEnd", 				"OnPublicEventUpdate", self)
	Apollo.RegisterEventHandler("ChallengeAbandon",				"OnChallengeUpdate", self)
	Apollo.RegisterEventHandler("ChallengeLeftArea", 			"OnChallengeUpdate", self)
	Apollo.RegisterEventHandler("ChallengeFailTime", 			"OnChallengeUpdate", self)
	Apollo.RegisterEventHandler("ChallengeFailArea", 			"OnChallengeUpdate", self)
	Apollo.RegisterEventHandler("ChallengeActivate",			"OnChallengeUpdate", self)
	Apollo.RegisterEventHandler("ChallengeCompleted", 			"OnChallengeUpdate", self)
	Apollo.RegisterEventHandler("ChallengeFailGeneric", 		"OnChallengeUpdate", self)
	
	
	-- Zone
	Apollo.RegisterEventHandler("VarChange_ZoneName", "OnZoneChange", self)
	Apollo.RegisterEventHandler("SubZoneChanged", "OnZoneChange", self)
	
	-- the rest
	self.xmlDoc = XmlDoc.CreateFromFile(cXmlFileName)
	self.xmlDoc:RegisterCallback("OnDocumentReady", self)
end

function ZenRadar:OnDocumentReady()
	self.wndMain = Apollo.LoadForm(self.xmlDoc, cMainWndClass, nil, self)
	self.wndMain:SetSizingMinimum(200,100)
	self.wndConfig = Apollo.LoadForm(self.xmlDoc, cCfgWndClass, nil, self)
	self.wndConfigBlackListTextBox = self.wndConfig:FindChild(cCfgWndTextBlackListFilter)
	self.wndConfigWhiteListTextBox = self.wndConfig:FindChild(cCfgWndTextWhiteListFilter)
	
	-- Modified by trackmaster as well
	self.wndConfigCheckEnableTrackMaster = self.wndConfig:FindChild(cCfgWndCheckEnableTrackMaster)
	
	if(cDeleteXml == true) then
		self.xmlDoc = nil
	end
	
	self.wndTrackedUnits = self.wndMain:FindChild(cMainWndTrackedUnits)
	self.wndSettingBtn = self.wndMain:FindChild(cMainWndSettingBtn)
	self.wndLockBtn = self.wndMain:FindChild(cMainWndLockBtn)
	self.wndWhiteBtn = self.wndMain:FindChild(cMainWndWhiteBtn)
	self.wndBlack = self.wndMain:FindChild(cMainWndBlackBtn)
	
	self.wndMain:Show(false, true)
	self.wndConfig:Show(false, true)
	
	Apollo.RegisterTimerHandler("OneSecTimer", "OnUpdate", self)
	if (cHasConfigurationButton == true) then
		Apollo.RegisterSlashCommand(cSlashCommand , "OnSlashCommand", self)
	end
end

-----------------------------------------------------------------------------------------------
-- Configuration
-----------------------------------------------------------------------------------------------
function ZenRadar:OnSlashCommand(strCommand, strParam)
	if(self:IsInValidState() == true) then
		strParam = strParam:lower()
		if (strParam == "toggle") then
			if(self.Settings[cHideRadarSetting] == false) then
				self:OnHideRadarCheck()
			else
				self:OnHideRadarUnCheck()
			end
		elseif (strParam == "lock") then
			if(self.Settings[cLockRadarSetting] == false) then
				self:OnRadarLockCheck()
			else
				self:OnRadarLockUncheck()
			end
		elseif (strParam == "wl") then
			self:OnAddWhiteList()
		elseif (strParam == "bl") then
			self:OnAddBlackList()
		else
			self:OnConfigure()
		end
	end
end

function ZenRadar:OnConfigure()
	self.wndConfig:Show(true, true)
	self:MatchConfigUiWithSettings()
end 

-----------------------------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------------------------
function ZenRadar:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then 
		return 
	end
	
	self.Settings[cUserSettingMainWindowLocation] = self.wndMain:GetLocation():ToTable()
	
	return self.Settings
end

function ZenRadar:OnRestore(eType, data)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then 
		return 
	end
	
	for property,tData in pairs(data) do
		self.Settings[property] = data[property]
	end
	
	-- Need to update the list settings now that the settings are loaded
	if(self.Settings[cBlackListFilterSetting] ~= nil) then
		self:ResetFilterArray(self.Settings[cBlackListFilterSetting], self.aBlackListFilters)
	end
	
	if(self.Settings[cWhiteListFilterSetting] ~= nil) then
		self:ResetFilterArray(self.Settings[cWhiteListFilterSetting], self.aWhiteListFilters)
	end
	
	if(self.wndMain ~= nil) then
		self:LoadSettings()
	else
		self.LoadSettingNeeded = true
	end
end

function ZenRadar:LoadSettings()
	tWindowLocation = WindowLocation.new(self.Settings[cUserSettingMainWindowLocation])
	self.wndMain:MoveToLocation(tWindowLocation)
	
	self:ResetFilteredList()
	self:MatchConfigUiWithSettings()
	self.LoadSettingNeeded = nil
end

-----------------------------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------------------------
function ZenRadar:UpdateFilterStringFromFilterList(filterTextBox, filterArray)
	local fullString = "";
	sortedArray = {}
	
	for key, unit in pairs(filterArray) do
		table.insert(sortedArray, key)
	end
	
	table.sort(sortedArray)
	
	for _, key in ipairs(sortedArray) do 
		fullString = fullString .. key .. " ; "
	end
	
	filterTextBox:SetText(fullString)
end

function ZenRadar:IsInValidState()
	return (self.player ~= nil 
		and self.player:IsValid() == true
		and self.wndMain ~= nil)
end

function ZenRadar:GetRowIndexOfId(id)
	local tRowIndex = nil
	if(self.wndTrackedUnits ~= nil) then
		local trackedUnitCount = self.wndTrackedUnits:GetRowCount()
		for i=1, trackedUnitCount,1 do
			-- SEDU - GetCellData returns nil always...bug?
			local rowId = tonumber(self.wndTrackedUnits:GetCellText(i, 4))
			if(rowId == id) then
				tRowIndex = i
				return tRowIndex
			end
		end
	end
	
	return nil
end

function ZenRadar:GetUnitFilterType(unit)
	local tUnitName = unit:GetName()
	local tUnitRewardInfo = unit:GetRewardInfo()
	local tActivation = unit:GetActivationState()
	local tVendorGroups = unit:GetVendorGroups()
	local tHarvestType = unit:GetHarvestRequiredTradeskillName()
	local tCanBeHarvested = unit:CanBeHarvestedBy(self.player)
	local affiliationName = unit:GetAffiliationName()

	unitDisplayType = nil
	
	if(self.aWhiteListFilters[tUnitName] ~= nil) then
		unitDisplayType  = cDisplayTypeWhiteList 
	end
	
	skipTest = false
	if(self.aBlackListFilters[tUnitName] ~= nil) then
		skipTest = true
	end

	if (skipTest == false 
		and unitDisplayType == nil
		and tActivation ~= nil 
		and (tActivation.Busy == nil or tActivation.Busy.bCanInteract == true)) then

		
		local nRewardInfoListCount = self:TableLength(tUnitRewardInfo)
		local isRewardful = false;
		if(tUnitRewardInfo ~= nil) then
			for idx = 1, nRewardInfoListCount do
				if (tUnitRewardInfo[idx].strType == "Challenge") then
					local bActiveChallenge = false
					local tAllChallenges = ChallengesLib.GetActiveChallengeList()
					
					for index, clgCurr in pairs(tAllChallenges) do
						if clgCurr ~= nil 
							and tUnitRewardInfo[idx].idChallenge == clgCurr:GetId() 
							and clgCurr:IsActivated() 
							and not clgCurr:IsInCooldown() 
							and not clgCurr:ShouldCollectReward()
							-- Biji use the stuff up there....does not work for completion weirdly
							-- will use objective values to determine completion
							and clgCurr:GetCurrentCount() < clgCurr:GetTotalCount()then
							bActiveChallenge = true
							break
						end
					end

					if bActiveChallenge then
						isRewardful = true
						break
					end
				elseif(tUnitRewardInfo[idx].splReward == nil) then
					isRewardful = true
					break
				end
			end
		end
		
		
		if(self.Settings[cShowTargetSetting] == true
			and (isRewardful == true
				or tActivation.QuestReward ~= nil)) then
				unitDisplayType = cDisplayTypeTarget
		elseif(self.Settings[cShowDataSetting] == true
			and tActivation.Datacube ~= nil) then
			unitDisplayType = cDisplayTypeLore
		elseif (self.Settings[cShowSurvivalSetting] == true
			and tHarvestType == L[cLocaleKeyHarvestSurvivalist]) then
			unitDisplayType = cDisplayTypeSurvivalist
		elseif (self.Settings[cShowRelicSetting] == true
			and tHarvestType == L[cLocaleKeyHarvestRelic]) then
			unitDisplayType = cDisplayTypeRelic
		elseif (self.Settings[cShowHerbSetting] == true
			and tHarvestType == L[cLocaleKeyHarvestFarming]) then
			unitDisplayType = cDisplayTypeFarming
		elseif (self.Settings[cShowMiningSetting] == true
			and tHarvestType == L[cLocaleKeyHarvestMining]) then
			unitDisplayType = cDisplayTypeMiner
		elseif (self.Settings[cShowHarvestSetting] == true
			and tCanBeHarvested == true) then
			unitDisplayType = tHarvestType
			
		elseif (self.Settings[cShowMailSetting] == true
			and tActivation.Mail ~= nil) then
			unitDisplayType = cDisplayTypeMail
		elseif (self.Settings[cShowSaveSetting] == true
			and tActivation.BindPoint ~= nil) then
			unitDisplayType = cDisplayTypeSave
		elseif (self.Settings[cShowTaxiSetting] == true
			and (tActivation.FlightPathSettler ~= nil
			or tActivation.FlightPathNew ~= nil
			or tActivation.FlightPath ~= nil
			or tActivation.InstancePortal ~= nil)) then
			unitDisplayType = cDisplayTypeTaxi
		elseif (self.Settings[cShowWorkBenchSetting] == true
			and tActivation.CraftingStation ~= nil) then
			unitDisplayType = cDisplayTypeWorkBench
		elseif (self.Settings[cShowCostumeSetting] == true
			and tActivation.Dye ~= nil) then
			unitDisplayType = cDisplayTypeCostume
		elseif (self.Settings[cShowBankSetting] == true
			and tActivation.Bank ~= nil) then
			unitDisplayType = cDisplayTypeBank
		elseif (self.Settings[cShowRepSetting] == true
			and tVendorGroups ~= nil
			and (self:TableVendorGroupIsType(tVendorGroups, "Liked")  == true
				or self:TableVendorGroupIsType(tVendorGroups, "Accepted") == true
				or self:TableVendorGroupIsType(tVendorGroups, "Popular") == true
				or self:TableVendorGroupIsType(tVendorGroups, "Esteemed") == true
				or self:TableVendorGroupIsType(tVendorGroups, "Beloved") == true)) then
			unitDisplayType = cDisplayTypeRep
		elseif (self.Settings[cShowRepSetting] == true
			and tVendorGroups ~= nil
			and (self:TableVendorGroupIsType(tVendorGroups, "Recipe") == true)) then
			unitDisplayType = cDisplayTypeRecipe
		elseif (self.Settings[cShowRepSetting] == true
			and tVendorGroups ~= nil
			and (self:TableVendorGroupIsType(tVendorGroups, "Housing") == true)) then
			unitDisplayType = cDisplayTypeHouse
		
		local cDisplayTypeHouse = "House"
local cDisplayTypeRecipe = "Recipe"
		elseif (self.Settings[cShowNewQuestSetting] == true
			and (tActivation.QuestNewMain ~= nil
				or tActivation.QuestNewTradeskill ~= nil)) then
			unitDisplayType = cDisplayTypeNewQuest
		elseif (self.Settings[cShowNewQuestOptSetting] == true
			and tActivation.QuestNew ~= nil) then
			unitDisplayType = cDisplayTypeNewQuest
		elseif (self.Settings[cShowDailySetting] == true
			and (tActivation.QuestNewRepeatable ~= nil
				or tActivation.QuestNewDaily)) then
			unitDisplayType = cDisplayTypeDailyQuest
			
		elseif (self.Settings[cShowScanSetting] == true
			and tActivation.ScientistScannable ~= nil
			and isRewardful == true) then
			unitDisplayType = cDisplayTypeScienceScan
		elseif (self.Settings[cShowRawSetting] == true
			and (tActivation.ScientistRawScannable ~= nil
				or tActivation.ScientistScannable ~= nil)) then
			unitDisplayType = cDisplayTypeScienceMat
		elseif (self.Settings[cShowExplorerSetting] == true
			and (tActivation.ExplorerInterest ~= nil
			or tActivation.ExplorerDoor ~= nil
			or tUnitName == L[cLocaleKeyNameExplorerDiscovery])) then
			unitDisplayType = cDisplayTypeExplorerDiscovery
		elseif (self.Settings[cShowExplorerTrailSetting] == true
			and tUnitName == L[cLocaleKeyNameExplorerTrail]) then
			unitDisplayType = cDisplayTypeExplorerTrail
		elseif (self.Settings[cShowSettlerSetting] == true
			and affiliationName == L[cLocaleKeyAffiliationSettlerResource]) then
			unitDisplayType = cDisplayTypeSettlerResource
		
		-- This is a last resort to send collectables as hidden....
		elseif (self.Settings[cShowCollectSetting] == true
			and tActivation.Collect ~= nil
			and unit:IsSettlerImprovement() == false
			and affiliationName ~= L[cLocaleKeyAffiliationSettlerResource]) then
			unitDisplayType = cDisplayTypeCollect
		end
	end
	
	return unitDisplayType
end

function ZenRadar:ResetFilteredList()
	if(self:IsInValidState() == true) then
		local tCurrentZoneInfo = GameLib.GetCurrentZoneMap()
		if tCurrentZoneInfo ~= nil then
			self.zoneName  = tCurrentZoneInfo.strName
		end
	
		self.aUnitsCheckValidityOnUpdate = {}
		self.wndTrackedUnits:DeleteAll()
		self.tmSelectedEntity = nil
		
		self.aAllQuests = {}
		self.aAllPathQuests = {}
		for key, unit in pairs(self.aAllUnits) do
			self.aUnitsToCheckOnNextUpdate[unit:GetId()] = unit
		end
		
		for idx, epiEpisode in pairs(QuestLib.GetTrackedEpisodes(true)) do
			for nIdx, queQuest in pairs(epiEpisode:GetTrackedQuests(0, true)) do
				self:ValidateFilterOnQuest(queQuest)
			end
		end
		
		local pepCurrent = PlayerPathLib:GetCurrentEpisode()
		if(pepCurrent ~= nil) then
			local tFullMissionList = pepCurrent:GetMissions()
			for _, pmMission in ipairs(tFullMissionList) do
				self:ValidateFilterOnPathQuest(pmMission)
			end
		end
	end
end
		
function ZenRadar:DistanceToUnit(unit)
	local posPlayer = self.player:GetPosition()
	local posTarget = unit:GetPosition()
	local nDeltaX = posTarget.x - posPlayer.x
	local nDeltaY = posTarget.y - posPlayer.y
	local nDeltaZ = posTarget.z - posPlayer.z
	
	return math.sqrt(math.pow(nDeltaX, 2) + math.pow(nDeltaY, 2) + math.pow(nDeltaZ, 2))
end 

function ZenRadar:GetClosestIndicator(queQuest)
	local posPlayer = self.player:GetPosition()
	distanceMin = nil
	local minIndicator = nil
	for idRegion, tRegion in pairs(queQuest:GetMapRegions()) do
		local indicator = tRegion.tIndicator
		local nDeltaX = indicator.x - posPlayer.x
		local nDeltaY = indicator.y - posPlayer.y
		local nDeltaZ = indicator.z - posPlayer.z
		distance = math.sqrt(math.pow(nDeltaX, 2) + math.pow(nDeltaY, 2) + math.pow(nDeltaZ, 2))
		
		if(distanceMin == nil or distance < distanceMin) then
			distanceMin = distance
			minIndicator = indicator
		end
	end
	
	return minIndicator
end

function ZenRadar:DistanceToQuest(queQuest)
	local distance = nil
	-- indicator can be nil when phasing out
	local indicator = self:GetClosestIndicator(queQuest)
	
	if(indicator ~= nil) then
		local posPlayer = self.player:GetPosition()
		
		local nDeltaX = indicator.x - posPlayer.x
		local nDeltaY = indicator.y - posPlayer.y
		local nDeltaZ = indicator.z - posPlayer.z
		distance = math.sqrt(math.pow(nDeltaX, 2) + math.pow(nDeltaY, 2) + math.pow(nDeltaZ, 2))
	end
	
	return distance
end

function ZenRadar:MatchConfigUiWithSettings()
	-- LUA limit of 60 upvalues...must stop uasing variables
	self.wndConfig:FindChild(cCfgWndCheckData):SetCheck(self.Settings[cShowDataSetting])
	self.wndConfig:FindChild(cCfgWndCheckCollect):SetCheck(self.Settings[cShowCollectSetting])
	self.wndConfig:FindChild(cCfgWndCheckMining):SetCheck(self.Settings[cShowMiningSetting])
	self.wndConfig:FindChild(cCfgWndCheckSurvival):SetCheck(self.Settings[cShowSurvivalSetting])
	self.wndConfig:FindChild(cCfgWndCheckRelic):SetCheck(self.Settings[cShowRelicSetting])
	self.wndConfig:FindChild(cCfgWndCheckHerb):SetCheck(self.Settings[cShowHerbSetting])
	self.wndConfig:FindChild(cCfgWndCheckHarvest):SetCheck(self.Settings[cShowHarvestSetting])
	self.wndConfig:FindChild(cCfgWndCheckScientistRaw):SetCheck(self.Settings[cShowRawSetting])
	
	self.wndConfig:FindChild(cCfgWndCheckMail):SetCheck(self.Settings[cShowMailSetting])
	self.wndConfig:FindChild(cCfgWndCheckSave):SetCheck(self.Settings[cShowSaveSetting])
	self.wndConfig:FindChild(cCfgWndCheckTaxi):SetCheck(self.Settings[cShowTaxiSetting])
	self.wndConfig:FindChild(cCfgWndCheckWorkBench):SetCheck(self.Settings[cShowWorkBenchSetting])
	self.wndConfig:FindChild(cCfgWndCheckCostume):SetCheck(self.Settings[cShowCostumeSetting])
	self.wndConfig:FindChild("ChBank"):SetCheck(self.Settings[cShowBankSetting])
	self.wndConfig:FindChild("ChRep"):SetCheck(self.Settings[cShowRepSetting])
	
	self.wndConfig:FindChild(cCfgWndCheckNewQuest):SetCheck(self.Settings[cShowNewQuestSetting])
	self.wndConfig:FindChild("ChNewQuestOpt"):SetCheck(self.Settings[cShowNewQuestOptSetting])
	self.wndConfig:FindChild(cCfgWndCheckQuestObj):SetCheck(self.Settings[cShowQuestObjSetting])
	self.wndConfig:FindChild(cCfgWndCheckPathQuestObj):SetCheck(self.Settings[cShowPathQuestObjSetting])
	self.wndConfig:FindChild(cCfgWndCheckDailyQuest):SetCheck(self.Settings[cShowDailySetting])
	self.wndConfig:FindChild("ChTarget"):SetCheck(self.Settings[cShowTargetSetting])
	self.wndConfig:FindChild(cCfgWndCheckScientist):SetCheck(self.Settings[cShowScanSetting])
	self.wndConfig:FindChild(cCfgWndCheckExplorer):SetCheck(self.Settings[cShowExplorerSetting])
	self.wndConfig:FindChild(cCfgWndCheckExplorerTrail):SetCheck(self.Settings[cShowExplorerTrailSetting])
	self.wndConfig:FindChild(cCfgWndCheckSettler):SetCheck(self.Settings[cShowSettlerSetting])
	
	self.wndConfig:FindChild(cCfgWndCheckAutoHideNoEntities):SetCheck(self.Settings[cAutoHideNoEntitiesSetting])
	self.wndConfig:FindChild(cCfgWndCheckAutoHideInCombat):SetCheck(self.Settings[cAutoHideInCombatSetting])
	self.wndConfig:FindChild("ChHideRadar"):SetCheck(self.Settings[cHideRadarSetting])
	self.wndConfig:FindChild("ChHideBtn"):SetCheck(self.Settings[cHideButtonsSetting])
	
	
	self.wndConfigCheckEnableTrackMaster:SetCheck(self.Settings[cEnableTrackMasterSetting])
	self.wndConfig:FindChild(cCfgWndCheckEnableTrackMasterClosest):SetCheck(self.Settings[cEnableTrackMasterClosestSetting])
	self.wndConfig:FindChild(cCfgWndCheckEnableTrackMasterSelected):SetCheck(self.Settings[cEnableTrackMasterSelectedSetting])
	
	self.wndMain:FindChild(cCfgWndCheckLockRadar):SetCheck(self.Settings[cLockRadarSetting])
	
	self:UpdateFilterStringFromFilterList(self.wndConfigWhiteListTextBox, self.aWhiteListFilters)
	self:UpdateFilterStringFromFilterList(self.wndConfigBlackListTextBox, self.aBlackListFilters)
	
	if(self.Settings[cLockRadarSetting] == true) then
		self:OnRadarLockCheck()
	end
	
	if(self.Settings[cHideButtonsSetting] == true) then
		self:OnHideButtonsCheck()
	end
end

function ZenRadar:TableLength(theTable)
	local length = 0;
	if(theTable ~= nil) then
		for key, value in pairs(theTable) do
			length = length + 1
		end
	end
	return length
end

function ZenRadar:TableClear(filterArray)
	for key, value in pairs(filterArray) do
		filterArray[key] = nil
	end
end

function ZenRadar:TableVendorGroupIsType(theTable, theValue)
	local returnedResult = false
	for _, value in pairs(theTable) do
		-- table values aren ot strings in lua
		if (string.find(value.strName, theValue) ~= nil) then
			returnedResult = true
			break
		end
	end
	return returnedResult
end
	
function ZenRadar:ResetFilterArray(stringFilter, filterArray)
	self:TableClear(filterArray)
	for x in string.gmatch(stringFilter, "[^;]+") do
		trimmedString = x:find'^%s*$' and '' or x:match'^%s*(.*%S)'
		if(trimmedString ~= '') then
			filterArray[trimmedString] = true
		end
	end
end

function ZenRadar:ValidateFilterOnQuest(queQuest)
	if(self.wndTrackedUnits ~= nil) then
		local questId = queQuest:GetId()
		if(self.Settings[cShowQuestObjSetting] == true
			and string.match(self.zoneName, queQuest:GetCategory():GetTitle())
			and self:GetClosestIndicator(queQuest) ~= nil) then
			self.aAllQuests[questId] = queQuest
			self:AddRow(questId, queQuest:GetTitle(), cDisplayTypeQuestObj, self:DistanceToQuest(queQuest))
		else
			self.aAllQuests[questId] = nil
			self:DeleteRow(questId)
		end
	end
end

function ZenRadar:ValidateFilterOnPathQuest(queQuest)
	if(self.wndTrackedUnits ~= nil) then
		local questId = queQuest:GetId()
		if(self.Settings[cShowPathQuestObjSetting] == true
			and queQuest:IsComplete() ~= true
			and queQuest:IsStarted() == true
			and queQuest:GetDistance() > 0
			and queQuest:GetDistance() < 9999) then
			self.aAllPathQuests[questId] = queQuest
			self:AddRow(questId, queQuest:GetName(), cDisplayTypePathQuestObj, queQuest:GetDistance())
		else
			self.aAllPathQuests[questId] = nil
			self:DeleteRow(questId)
		end
	end
end

function ZenRadar:ValidateFilterOnUnit(unit)
	if(self.wndTrackedUnits ~= nil) then
		local unitId = unit:GetId()
		local unitType = self:GetUnitFilterType(unit)
		if(unitType ~= nil) then
			local distance = self:DistanceToUnit(unit)
			local skipTurn = false
			if(unitType == cDisplayTypeExplorerTrail) then
				self.aUnitsCheckValidityOnUpdate[unitId] = unit	
				if(distance < cDistanceMinExplorerTrail) then
					skipTurn = true
				end
			end
			if(skipTurn ~= true) then
				self:AddRow(unitId, unit:GetName(), unitType, distance)
			end
		else
			self:DeleteRow(unitId)
			if(self.tmSelectedEntity == unit) then
				self.tmSelectedEntity = nil
			end
		end
	end
end

function ZenRadar:AddRow(id, name, theType, distance)
	local tRowIndex = self:GetRowIndexOfId(id)
	
	if(tRowIndex == nil) then
		local tRow = self.wndTrackedUnits:AddRow("")
		self.wndTrackedUnits:SetCellText(tRow, 1, name)
		self.wndTrackedUnits:SetCellText(tRow, 2, theType)
		self.wndTrackedUnits:SetCellText(tRow, 3, string.format("%5d m", distance))
		self.wndTrackedUnits:SetCellText(tRow, 4, id)
	end
end

function ZenRadar:DeleteRow(id)
	local tRowIndex = self:GetRowIndexOfId(id)
	
	if(tRowIndex ~= nil) then
		self.wndTrackedUnits:DeleteRow(tRowIndex)
	end
end

function ZenRadar:RetrieveEntityById(id)
	local entity = nil
	
	local unit = self.aAllUnits[id]
	if(unit ~= nil) then
		entity = unit
	end
	
	local quest = self.aAllQuests[id]
	if(quest ~= nil) then
		entity = quest
	end
	
	local quest = self.aAllPathQuests[id]
	if(quest ~= nil) then
		entity = quest
	end
	
	return entity
end

-----------------------------------------------------------------------------------------------
-- Events
-----------------------------------------------------------------------------------------------
function ZenRadar:OnUpdate()
	--SEDU - stability during zone changes
	self.previousPlayer = self.player
	self.player = GameLib.GetPlayerUnit()
	if (self:IsInValidState() == false) then
		self.player = nil
	elseif (self.previousPlayer ~= self.player) then
		self:ResetFilteredList()
	end
	-- Player can be nil sometime...no update then
	if(self:IsInValidState() == true) then
		if(self.LoadSettingNeeded == true) then
			self:LoadSettings()
		end
		
		for key, unit in pairs(self.aUnitsToCheckOnNextUpdate) do
			self:ValidateFilterOnUnit(unit)
			self.aUnitsToCheckOnNextUpdate[unit:GetId()] = nil
		end
		
		for key, unit in pairs(self.aUnitsCheckValidityOnUpdate) do
			local unitType = self:GetUnitFilterType(unit)
			local unitId = unit:GetId()
			local distance = self:DistanceToUnit(unit)
			if(unitType == cDisplayTypeExplorerTrail
				and distance < cDistanceMinExplorerTrail) then
				self.aUnitsCheckValidityOnUpdate[unitId] = nil
				if(self.tmSelectedEntity == unit) then
					self.tmSelectedEntity = nil
				end
				self:DeleteRow(unitId)
				
				self:ValidateFilterOnUnit(unit)
			end
		end
		
		local trackedUnitCount = self.wndTrackedUnits:GetRowCount()
		local unitsToDelete = {}
		for i=1, trackedUnitCount,1 do
			-- SEDU - GetCellData returns nil always...bug?
			local id = tonumber(self.wndTrackedUnits:GetCellText(i, 4))
			local distance = nil
			
			local unit = self.aAllUnits[id]
			if(unit ~= nil) then
				-- Remove dead units
				if(unit:IsDead() ~= true) then
					distance = self:DistanceToUnit(unit)
				else
					unitsToDelete[unit:GetId()] = unit
				end
			end
			
			local quest = self.aAllQuests[id]
			if(quest ~= nil) then
				distance = self:DistanceToQuest(quest)
			end
			
			local quest = self.aAllPathQuests[id]
			if(quest ~= nil) then
				distance = quest:GetDistance()
			end
			
			-- Some quests get nil distances when you zone out
			if(distance ~= nil) then
				self.wndTrackedUnits:SetCellText(i, 3, string.format("%5d m", distance))
			end
		end
		
		
		for key, unit in pairs(unitsToDelete) do
			self:OnUnitDestroyed(unit)
		end
		trackedUnitCount = self.wndTrackedUnits:GetRowCount()
		
		-- Sort by distance
		self.wndTrackedUnits:SetSortColumn(3, true)
		
		if(self.tmSelectedEntity ~= nil) then
			local index = self:GetRowIndexOfId(self.tmSelectedEntity:GetId())
			self.wndTrackedUnits.SetCurrentRow(self.wndTrackedUnits, index)
		else
			self.wndTrackedUnits.SetCurrentRow(self.wndTrackedUnits, 0)
		end
		
		if (self.Settings[cHideRadarSetting] == false) then
			if((self.player:IsInCombat() == false
					or self.Settings[cAutoHideInCombatSetting] == false)
				and (trackedUnitCount > 0
					or self.Settings[cAutoHideNoEntitiesSetting] == false)) then
				self.wndMain:Show(true)
			else
				self.wndMain:Show(false)
			end
		else
			self.wndMain:Show(false)
		end
		
		
		-- TrackMaster Part
		if(self.Settings[cEnableTrackMasterSetting] == true) then
			if(self.tm == nil) then
				self.tm = Apollo.GetAddon("TrackMaster")
				if(self.tm ~= nil) then
					
					if(self.Settings[cTrackMasterLineNo] == nil) then
						self.Settings[cTrackMasterLineNo] = 1
					end
					
					self.tmConfig = self.tm:AddToConfigMenu(self.tm.Type.Track, "ZenRadar", {
						CanFire = false,
						CanEnable = true,
						IsChecked = self.Settings[cEnableTrackMasterSetting],
						OnEnableChanged = (function(isEnabled)
							self.Settings[cEnableTrackMasterSetting] = isEnabled
							self.wndConfigCheckEnableTrackMaster:SetCheck(self.Settings[cEnableTrackMasterSetting])
						end),
						LineNo = self.Settings[cTrackMasterLineNo],
						OnLineChanged = (function(lineNo)
							self.Settings[cTrackMasterLineNo] = lineNo
						end)
					})
				end
			else
				if(self.Settings[cEnableTrackMasterSelectedSetting] == true
					and self.tmSelectedEntity ~= nil) then
					self.tmTrackedEntity = self.tmSelectedEntity
				elseif(self.Settings[cEnableTrackMasterClosestSetting] == true
					and trackedUnitCount > 0) then
					-- SEDU - GetCellData returns nil always...bug?
					local id = tonumber(self.wndTrackedUnits:GetCellText(1, 4))
					self.tmTrackedEntity = self:RetrieveEntityById(id)
				else
					self.tmTrackedEntity = nil
				end
				if(self.tmTrackedEntity == nil) then
					self.tm:SetTarget(nil, -1, self.Settings[cTrackMasterLineNo])
				elseif(self.tmTrackedEntity.GetActivationState ~= nil) then
					self.tm:SetTarget(self.tmTrackedEntity, -1, self.Settings[cTrackMasterLineNo])
				else
					local indicator = self:GetClosestIndicator(self.tmTrackedEntity)
					if(indicator ~= nil) then
						local vector = Vector3.New(indicator.x,indicator.y,indicator.z)
						self.tm:SetTarget(vector, -1, self.Settings[cTrackMasterLineNo])
					else
						self.tm:SetTarget(nil, -1, self.Settings[cTrackMasterLineNo])
					end
				end
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
-- Apollo Events
---------------------------------------------------------------------------------------------------
function ZenRadar:OnUnitCreated(unit)
	local unitId = unit:GetId()
	self.aAllUnits[unitId] = unit	
	self.aUnitsToCheckOnNextUpdate[unitId] = unit
end

function ZenRadar:OnUnitDestroyed(unit)
	local unitId = unit:GetId();
	self.aAllUnits[unitId] = nil
	self.aUnitsToCheckOnNextUpdate[unitId] = nil
	self.aUnitsCheckValidityOnUpdate[unitId] = nil
	
	if(self.tmSelectedEntity == unit) then
		self.tmSelectedEntity = nil
	end
	if(self:IsInValidState() == true) then
		self:DeleteRow(unitId)
	end
end

function ZenRadar:OnUnitChanged(unit)
	if(self:IsInValidState() == true) then
		self.aUnitsToCheckOnNextUpdate[unit:GetId()] = unit
	end
end

function ZenRadar:OnUnitCombat(unit)
	if(self:IsInValidState() == true) then
		if(unit == self.player
			and self.Settings[cAutoHideInCombatSetting] == true) then
			self.wndMain:Show(false)
		end
	end
end

function ZenRadar:OnQuestStateChanged(queQuest, eState)
	if(self:IsInValidState() == true) then
		local questId = queQuest:GetId()
		if(eState == Quest.QuestState_Accepted) then
			self:ValidateFilterOnQuest(queQuest)
		elseif(eState == Quest.QuestState_Achieved
			or eState == Quest.QuestState_Completed
			or eState == Quest.QuestState_Botched
			or eState == Quest.QuestState_Abandonned
			or eState == Quest.QuestState_Unknown
			or eState == Quest.QuestState_Ignored) then
			self.aAllQuests[questId] = nil
			if(self.tmSelectedEntity == queQuest) then
				self.tmSelectedEntity = nil
			end
			self:DeleteRow(questId)
		end
		self:ResetFilteredList()
	end
end

function ZenRadar:OnQuestObjectiveUpdated(queQuest, eState)
	if(self:IsInValidState() == true) then
		self:ValidateFilterOnQuest(queQuest)
	end
	self:ResetFilteredList()
end

function ZenRadar:OnPlayerPathMissionActivate(queQuest)
	if(self:IsInValidState() == true) then
		self:ValidateFilterOnPathQuest(queQuest)
		self:ResetFilteredList()
	end
end

function ZenRadar:OnPlayerPathMissionUpdate(queQuest)
	if(self:IsInValidState() == true) then
		self:ResetFilteredList()
	end
end

function ZenRadar:OnPlayerPathMissionDeactivate(queQuest)
	if(self:IsInValidState() == true) then
		local questId = queQuest:GetId()
		self.aAllPathQuests[questId] = nil
		if(self.tmSelectedEntity == queQuest) then
			self.tmSelectedEntity = nil
		end
		self:DeleteRow(questId)
		self:ResetFilteredList()
	end
end

function ZenRadar:OnPlayerPathMissionComplete(queQuest)
	if(self:IsInValidState() == true) then
		local questId = queQuest:GetId()
		self.aAllPathQuests[questId] = nil
		if(self.tmSelectedEntity == queQuest) then
			self.tmSelectedEntity = nil
		end
		self:DeleteRow(questId)
		self:ResetFilteredList()
	end
end

function ZenRadar:OnPublicEventUpdate(queQuest)
	if(self:IsInValidState() == true) then
		self:ResetFilteredList()
	end
end

function ZenRadar:OnChallengeUpdate(challengeId)
	if(self:IsInValidState() == true) then
		self:ResetFilteredList()
	end
end

function ZenRadar:OnZoneChange(oVar, strNewZone)
	local tCurrentZoneInfo = GameLib.GetCurrentZoneMap()
	if (tCurrentZoneInfo == nil
		or self.zoneName ~= tCurrentZoneInfo.strName) then
		-- force a reset
		self.player = nil
	end
end

---------------------------------------------------------------------------------------------------
-- UI Events
---------------------------------------------------------------------------------------------------
function ZenRadar:OnTrackedItemPick(wndControl, wndHandler, iRow, iCol)
	-- SEDU - GetCellData returns nil always...bug?
	local id = tonumber(self.wndTrackedUnits:GetCellText(iRow, 4))
	local retrievedEntity =  self:RetrieveEntityById(id)
	if(self.tmSelectedEntity ~= retrievedEntity) then
		self.tmSelectedEntity = retrievedEntity
		self.tmSelectedEntity:ShowHintArrow()
	else
		self.tmSelectedEntity = nil
	end
end

function ZenRadar:OnConfigClose( wndHandler, wndControl, eMouseButton )
	self.wndConfig:Show(false, true)
end

function ZenRadar:OnDataCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowDataSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnDataUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowDataSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnCollectCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowCollectSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnCollectUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowCollectSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnMiningCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowMiningSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnMiningUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowMiningSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnSurvivalCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowSurvivalSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnSurvivalUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowSurvivalSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnRelicCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowRelicSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnRelicUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowRelicSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnHerbCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowHerbSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnHerbUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowHerbSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnHarvestCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowHarvestSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnHarvestUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowHarvestSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnRawCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowRawSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnRawUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowRawSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnMailCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowMailSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnMailUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowMailSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnSaveCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowSaveSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnSaveUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowSaveSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnTaxiCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowTaxiSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnTaxiUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowTaxiSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnWorkBenchCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowWorkBenchSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnWorkBenchUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowWorkBenchSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnCostumeCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowCostumeSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnCostumeUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowCostumeSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnBankCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowBankSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnBankUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowBankSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnRepCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowRepSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnRepUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowRepSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnNewQuestCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowNewQuestSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnNewQuestUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowNewQuestSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnNewQuestOptCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowNewQuestOptSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnNewQuestOptUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowNewQuestOptSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnTargetCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowTargetSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnTargetUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowTargetSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnQuestObjCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowQuestObjSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnQuestObjUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowQuestObjSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnPathQuestObjCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowPathQuestObjSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnPathQuestObjUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowPathQuestObjSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnDailyCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowDailySetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnDailyUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowDailySetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnScanCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowScanSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnScanUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowScanSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnExplorerCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowExplorerSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnExplorerUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowExplorerSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnExplorerTrailCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowExplorerTrailSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnExplorerTrailUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowExplorerTrailSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnSettlerCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowSettlerSetting] = true
	self:ResetFilteredList()
end

function ZenRadar:OnSettlerUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cShowSettlerSetting] = false
	self:ResetFilteredList()
end

function ZenRadar:OnAutoHideNoEntitesCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cAutoHideNoEntitiesSetting] = true
end

function ZenRadar:OnAutoHideNoEntitesUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cAutoHideNoEntitiesSetting] = false
end

function ZenRadar:OnAutoHideInCombatCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cAutoHideInCombatSetting] = true
end

function ZenRadar:OnAutoHideInCombatUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cAutoHideInCombatSetting] = false
end

function ZenRadar:OnHideRadarCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cHideRadarSetting] = true
end

function ZenRadar:OnHideRadarUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cHideRadarSetting] = false
end

function ZenRadar:OnSaveBlackListFilter( wndHandler, wndControl, eMouseButton )
	self:ResetFilterArray(self.wndConfigBlackListTextBox:GetText(), self.aBlackListFilters)
	self.Settings[cBlackListFilterSetting] = self.wndConfigBlackListTextBox:GetText()
	
	self:UpdateFilterStringFromFilterList(self.wndConfigBlackListTextBox, self.aBlackListFilters)
	self:ResetFilteredList()
end

function ZenRadar:OnSaveWhiteListFilter( wndHandler, wndControl, eMouseButton )
	self:ResetFilterArray(self.wndConfigWhiteListTextBox:GetText(), self.aWhiteListFilters)
	self.Settings[cWhiteListFilterSetting] = self.wndConfigWhiteListTextBox:GetText()
	
	self:UpdateFilterStringFromFilterList(self.wndConfigWhiteListTextBox, self.aWhiteListFilters)
	self:ResetFilteredList()
end

function ZenRadar:OnRadarLockCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cLockRadarSetting] = true
	self.wndMain:SetStyle("Moveable", false)
	self.wndMain:SetStyle("Sizable", false)
	-- Might be called by macros
	self.wndMain:FindChild(cCfgWndCheckLockRadar):SetCheck(self.Settings[cLockRadarSetting])
end

function ZenRadar:OnRadarLockUncheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cLockRadarSetting] = false
	self.wndMain:SetStyle("Moveable", true)
	self.wndMain:SetStyle("Sizable", true)
	-- Might be called by macros
	self.wndMain:FindChild(cCfgWndCheckLockRadar):SetCheck(self.Settings[cLockRadarSetting])
end

function ZenRadar:OnAddBlackList( wndHandler, wndControl, eMouseButton )
	if(self.tmSelectedEntity ~= nil 
		and self.tmSelectedEntity.GetActivationState ~= nil) then
		local tTargetName = self.tmSelectedEntity:GetName()
		if(self.aWhiteListFilters[tTargetName] ~= nil) then
			self.aWhiteListFilters[tTargetName] = nil
			self:UpdateFilterStringFromFilterList(self.wndConfigWhiteListTextBox, self.aWhiteListFilters)
			self.Settings[cWhiteListFilterSetting] = self.wndConfigWhiteListTextBox:GetText()
			self:ResetFilteredList()
		elseif(self.aBlackListFilters[tTargetName] == nil) then
			self.aBlackListFilters[tTargetName] = true
			self.tmSelectedEntity = nil
			self:UpdateFilterStringFromFilterList(self.wndConfigBlackListTextBox, self.aBlackListFilters)
			self.Settings[cBlackListFilterSetting] = self.wndConfigBlackListTextBox:GetText()
			self:ResetFilteredList()
		end
	end	
end

function ZenRadar:OnAddWhiteList( wndHandler, wndControl, eMouseButton )
	local target = self.player:GetTarget()
	
	if(target ~= nil) then
		local tTargetName = target:GetName()
		if(self.aBlackListFilters[tTargetName] ~= nil) then
			self.aBlackListFilters[tTargetName] = nil
			self:UpdateFilterStringFromFilterList(self.wndConfigBlackListTextBox, self.aBlackListFilters)
			self.Settings[cBlackListFilterSetting] = self.wndConfigBlackListTextBox:GetText()
			self:ResetFilteredList()
		elseif(self.aWhiteListFilters[tTargetName] == nil) then
			self.aWhiteListFilters[tTargetName] = true
			self:UpdateFilterStringFromFilterList(self.wndConfigWhiteListTextBox, self.aWhiteListFilters)
			self.Settings[cWhiteListFilterSetting] = self.wndConfigWhiteListTextBox:GetText()
			self:ResetFilteredList()
		end
	end	
end

function ZenRadar:OnHideButtonsCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cHideButtonsSetting] = true
	self.wndSettingBtn:Show(false)
	self.wndLockBtn:Show(false)
	self.wndWhiteBtn:Show(false)
	self.wndBlack:Show(false)
	self.wndTrackedUnits:SetAnchorOffsets(0, 3, 0, -4)
end

function ZenRadar:OnHideButtonsUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cHideButtonsSetting] = false
	self.wndSettingBtn:Show(true)
	self.wndLockBtn:Show(true)
	self.wndWhiteBtn:Show(true)
	self.wndBlack:Show(true)
	self.wndTrackedUnits:SetAnchorOffsets(0, 3, -26, -4)
end

function ZenRadar:OnEnableTrackMasterCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cEnableTrackMasterSetting] = true
	if(self.tmConfig ~= nil) then
		self.tmConfig:SetEnabled(self.Settings[cEnableTrackMasterSetting])
	end
end

function ZenRadar:OnEnableTrackMasterUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cEnableTrackMasterSetting] = false
	if(self.tm ~= nil) then
		if(self.tmTrackedEntity ~= nil) then
			self.tm:SetTarget(nil, -1, self.Settings[cTrackMasterLineNo])
		end
		self.tmConfig:SetEnabled(self.Settings[cEnableTrackMasterSetting])
	end
end

function ZenRadar:OnEnableTrackMasterClosestCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cEnableTrackMasterClosestSetting] = true
end

function ZenRadar:OnEnableTrackMasterClosestUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cEnableTrackMasterClosestSetting] = false
end

function ZenRadar:OnEnableTrackMasterSelectedCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cEnableTrackMasterSelectedSetting] = true
end

function ZenRadar:OnEnableTrackMasterSelectedUnCheck( wndHandler, wndControl, eMouseButton )
	self.Settings[cEnableTrackMasterSelectedSetting] = false
end


-----------------------------------------------------------------------------------------------
-- Instance
-----------------------------------------------------------------------------------------------
local ZenRadarInst = ZenRadar:new()
ZenRadarInst:Init()
